#include <iostream>
#include "linear_programming.h"
#include "basic_output.h"



//extension of namespace lp - making it sort of private
//-----------------------------------------------------
namespace lp
{
	const pos_int_type ITERATION_LIMIT(50);

	real_matrix_type T(0,0);
			
	pos_int_vector_type basis(0);
	pos_int_vector_type non_basis(0);

	pos_int_type pivot_line(0);
	pos_int_type pivot_column(0);

	bool phase_1(true);

	real_vector_type _results(0);
	real_type _max(0);

	
	void initialize();
	void pivot();
	void iterate_normal();
	void iterate_degeneracy();
	void terminate_phase_1();
	

	//instanciation of global variables	
	real_matrix_type A(0,0);
	real_vector_type b(0);
	real_vector_type c(0);	
}
//-----------------------------------------------------



//implementation of namespace lp
//-----------------------------------------------------
void lp::simplex()
{
	initialize();

	pivot_line = 0;
	real_type min(T[0][0]);

	for(pos_int_type i(0); i < basis.size(); ++i)
	{
		if(T[i][0] < min)
		{
			pivot_line = i;
			min = T[i][0];
		}
	}

	//resovling phase_1 first
	if(min < 0)
	{
		#ifdef PRINT_ALL
		output << "STARTING PHASE 1 WITH TABLEAU" << endl;	
		print_tableau();
		output << "------------------------------" << endl << endl;
		#endif
		
		pivot_column = (T[0].size() - 1);
		pivot();

		try
		{
			iterate_normal();
		}
		catch(Clp_error e)
		{			
			if(e.state == OPTIMAL)
			{					
				if(T.back()[0] == 0)
				{					
					//since there are basic feasible solutions we go to phase 2
					terminate_phase_1();
					
					#ifdef PRINT_ALL
					output << "AND NOW STARTING PHASE 2 WITH TABLEAU" << endl;	
					print_tableau();
					output << "------------------------------" << endl << endl;
					#endif
					
					iterate_normal();
				}
				else
				{					
					throw Clp_error(NOT_FEASIBLE);
				}
			}
			else
			{				
				throw e;
			}
		}
	}
	//going directly to phase_2
	else
	{
		terminate_phase_1();

		#ifdef PRINT_ALL
		output << "STARTING DIRECTLY PHASE 2 WITH TABLEAU" << endl;	
		print_tableau();
		output << "------------------------------" << endl << endl;
		#endif
					
		iterate_normal();
	}
}



//show (with a written example) what the form of the tableau is after initialization	
void lp::initialize()
{
	if((A.size() == 0) || (b.size() == 0) || (c.size() == 0) || (b.size() != A.size()) || (c.size() != A[0].size()))
	{
		throw Clp_error(INPUT_VARIABLES);
	}

	
	basis.resize(A.size());
	non_basis.resize(A[0].size() + 1);
	T.resize(A.size() + 2, A[0].size() + 2);

	phase_1 = true;
	
	pivot_line = 0;
	pivot_column = 0;

	//output << A << endl;
	//output << A.size() << endl;
	
	for(pos_int_type i(0); i < A.size(); ++i)
	{
		T[i][0] = b[i];
		T[i].back() = 1;

		basis[i] = i + A[0].size() + 1;

		for(pos_int_type j(0); j < A[0].size(); ++j)
		{
			T[i][j + 1] = -A[i][j];
		}
	}

	for(pos_int_type j(0); j < c.size(); ++j)
	{
		T[T.size() - 2][j + 1] = c[j];
		T.back()[j + 1] = 0;

		non_basis[j] = j + 1;
	}

	non_basis.back() = 0;

	T[T.size() - 2][0] = 0;
	T[T.size() - 2].back() = 0;
	T.back()[0] = 0;
	T.back().back() = -1;
}



void lp::pivot()
{
	//(1) pivot variables - basis <-> non_basis

	int tmp(non_basis[pivot_column - 1]);
	non_basis[pivot_column - 1] = basis[pivot_line];
	basis[pivot_line] = tmp;

	
	//(2) pivot tableau
	T.pivot(pivot_line, pivot_column);
}



void lp::iterate_normal()
{	
	pos_int_type iteration_count(0);
	bool iterating(true);

	while(iterating)
	{		
		#ifdef PRINT_ALL
		output << "ITERATING - ";
		if(phase_1)
		{
			output << "PHASE 1" << endl;
		}
		else
		{
			output << "PHASE 2" << endl;
		}
		print_tableau();
		output << "------------------------------" << endl << endl;
		#endif

		//(1) Find pivot column - variable entering the basis

		pos_int_type j(1);

		while((j < T[0].size()) && (T.back()[j] <= 0))
		{
			++j;
		}
	
		if(j == T[0].size())
		{
			throw Clp_error(OPTIMAL);
		}
		else
		{
			pivot_column = j;
		}


		//(2) Find pivot line - variable leaving the basis
	
		real_type min(0);
		real_type temp(0);
		bool unbounded(true);

		for(pos_int_type i(0); i < basis.size(); ++i)
		{
			if(T[i][pivot_column] < 0)
			{
				if(unbounded)
				{
					unbounded = false;
					min = -(T[i][0] / T[i][pivot_column]);
					pivot_line = i;
				}
				else
				{
					temp = -(T[i][0] / T[i][pivot_column]);
					if(temp < min)
					{
						min = temp;
						pivot_line = i;
					}
				}
			}
		}

		if(unbounded)
		{
			throw Clp_error(UNBOUNDED);
		}

		//we must check if the base variable x_0 competes for leaving the basis (only on phase 1)
		if(phase_1)
		{
			pos_int_type index_zero(0);
			pos_int_type i(0);
	
			while((i < basis.size()) && (basis[i] != 0))
			{
				++i;
			}

			if(i == basis.size())
			{
				throw Clp_error(CRITICAL,"During phase 1 x_0 must be a basis variable.");
			}
			else
			{
				index_zero = i;
			}

			if((T[index_zero][pivot_column] < 0) && (-(T[index_zero][0] / T[index_zero][pivot_column]) == min))
			{
				pivot_line = index_zero;
			}
		}


		//(3) Do the pivot and check for degeneracy

		real_type z(T.back()[0]);

		pivot();

		if(T.back()[0] == z)
		{
			if(iteration_count < ITERATION_LIMIT)
			{
				iteration_count++;
			}
			else
			{
				iteration_count = 0;
				iterating = false;
				iterate_degeneracy();
			}
		}
		else
		{
			iteration_count = 0;
		}
	}
}



void lp::iterate_degeneracy()
{	
	bool iterating(true);

	while(iterating)
	{		
	 	//(1) Find pivot column with Bland's rule
	
		pos_int_type min_subscript(basis.size() + non_basis.size() + 1);
	
		for(pos_int_type j(1); j < T[0].size(); ++j)
		{
			if((T.back()[j] > 0) && (non_basis[j - 1] < min_subscript))
			{
				min_subscript = non_basis[j - 1];
				pivot_column = j;				
			}
		}

		if(min_subscript == (basis.size() + non_basis.size() + 1))
		{
			//one version
			//iterating = false;
			//iterate_normal();
			
			//other version
			throw Clp_error(OPTIMAL);
		}


		//(2) Find pivot line with Bland's rule
	
		real_type min(0);
		real_type temp(0);
		bool unbounded(true);
	
		min_subscript = basis.size() + non_basis.size() + 1;
	
		for(pos_int_type i(0); i < basis.size(); ++i)
		{
			if(T[i][pivot_column] < 0)
			{
				if(unbounded)
				{
					unbounded = false;
					min = -(T[i][0] / T[i][pivot_column]);
					pivot_line = i;
				}
				else
				{
					temp = -(T[i][0] / T[i][pivot_column]);
					if((temp < min) || ((temp == min) && (basis[i] < min_subscript)))
					{
						min = temp;
						min_subscript = basis[i];
						pivot_line = i;
					}
				}
			}
		}

		if(unbounded)
		{
			throw Clp_error(UNBOUNDED);
		}


		//(3) Do the pivot and check for further degeneracy

		real_type z(T.back()[0]);
	
		pivot();
	
		if(T.back()[0] != z)
		{
			iterating = false;
			iterate_normal();
		}
	}
}



void lp::terminate_phase_1()
{
	T.erase((T.end() - 1));

	pos_int_type index_zero(0);
	pos_int_type j(0);

	while((j < non_basis.size()) && (non_basis[j] != 0))
	{
		++j;
	}
				
	if(j == non_basis.size())
	{
		throw Clp_error(CRITICAL,"x_0 must be a non basis variable at the end of phase 1.");
	}

	index_zero = j;
	non_basis.erase(non_basis.begin() + index_zero);

	for(pos_int_type i(0); i < T.size(); ++i)
	{
		T[i].erase(T[i].begin() + index_zero + 1);
	}

	phase_1 = false;
}



void lp::print_tableau()
{
	for(pos_int_type i(0); i < T.size(); ++i)
	{
		for(pos_int_type j(0); j < T[0].size(); ++j)
		{
			output << T[i][j] << '\t';
		}
		output << endl;
	}
	output << endl;

	output << "Basis variables:" << endl;
	for(pos_int_type i(0); i < basis.size(); ++i)
	{
		output << "x_" << basis[i] << '\t';
	}
	output << endl;

	output << "Non-basis variables:" << endl;
	for(pos_int_type i(0); i < non_basis.size(); ++i)
	{
		output << "x_" << non_basis[i] << '\t';
	}
	output << endl << endl;
}



const real_vector_type& lp::results()
{
	_results.resize((T[0].size() - 1));

	pos_int_type i;
	for(pos_int_type k(1); k < T[0].size(); k++)
	{
		i = 0;
		while((i < basis.size()) && (basis[i] != k))
		{
			++i;
		}

		if(i == basis.size())
		{
			_results[k - 1] = 0;
		}
		else
		{
			_results[k - 1] = T[i][0];
		}
	}

	return _results;
}	
	
	

const real_type& lp::max()
{
	_max = T.back()[0];

	return _max;
}
	

